Imports Microsoft.VisualBasic
Imports System
Imports System.Web
Imports System.Workflow.Runtime
Imports System.Diagnostics
Imports System.Workflow.Runtime.Hosting
Imports WorkflowCore.Logging
Imports System.Collections.Generic
Imports System.Configuration

'////////////////////////////////////////////////////////////////////
' Created by Michael Kennedy
' http://www.michaelckennedy.net
' Copyright Michael Kennedy 2008.
' Part of the HRWorkflow sample application featured in MSDN Magazine.
'

Namespace HRWorkflow
	Public Class [Global]
		Inherits HttpApplication
        Private Shared privateWorkflowRuntime As WorkflowRuntime
		Public Shared Property WorkflowRuntime() As WorkflowRuntime
			Get
				Return privateWorkflowRuntime
			End Get
			Set(ByVal value As WorkflowRuntime)
				privateWorkflowRuntime = value
			End Set
		End Property
		Public Shared timers As Dictionary(Of Guid, Stopwatch) = New Dictionary(Of Guid, Stopwatch)()

		Protected Sub Application_Start(ByVal sender As Object, ByVal e As EventArgs)
			WorkflowRuntime = New WorkflowRuntime()

			InstallPersistenceService()

			AddHandler WorkflowRuntime.WorkflowCompleted, AddressOf WorkflowRuntime_WorkflowCompleted
			AddHandler WorkflowRuntime.WorkflowTerminated, AddressOf WorkflowRuntime_WorkflowTerminated
			AddHandler WorkflowRuntime.WorkflowIdled, AddressOf WorkflowRuntime_WorkflowIdled
			AddHandler WorkflowRuntime.WorkflowResumed, AddressOf WorkflowRuntime_WorkflowResumed
			AddHandler WorkflowRuntime.WorkflowStarted, AddressOf WorkflowRuntime_WorkflowStarted
			AddHandler WorkflowRuntime.WorkflowPersisted, AddressOf WorkflowRuntime_WorkflowPersisted
			AddHandler WorkflowRuntime.WorkflowLoaded, AddressOf WorkflowRuntime_WorkflowLoaded

			WorkflowRuntime.StartRuntime()

			TraceDT.Add("Global: Creating and starting WorkflowRuntime.", Guid.Empty, Me.GetType())
		End Sub

		Private Sub WorkflowRuntime_WorkflowLoaded(ByVal sender As Object, ByVal e As WorkflowEventArgs)
			TraceDT.Add("Global: Workflow instance restored from database.", e.WorkflowInstance.InstanceId, Me.GetType())
		End Sub

		Private Sub WorkflowRuntime_WorkflowPersisted(ByVal sender As Object, ByVal e As WorkflowEventArgs)
			TraceDT.Add("Global: Workflow instance unloaded and persisted to database.", e.WorkflowInstance.InstanceId, Me.GetType())
		End Sub

		Private Sub WorkflowRuntime_WorkflowStarted(ByVal sender As Object, ByVal e As WorkflowEventArgs)
			TraceDT.Add("Global: Workflow instance started.", e.WorkflowInstance.InstanceId, Me.GetType())

			Dim watch As Stopwatch = GetTimerByWorkflow(e.WorkflowInstance.InstanceId)
			watch.Start()
		End Sub

		Private Sub WorkflowRuntime_WorkflowResumed(ByVal sender As Object, ByVal e As WorkflowEventArgs)
			TraceDT.Add("Global: Workflow instance resumed.", e.WorkflowInstance.InstanceId, Me.GetType())
			Dim watch As Stopwatch = GetTimerByWorkflow(e.WorkflowInstance.InstanceId)
			watch.Start()
		End Sub

		Private Sub WorkflowRuntime_WorkflowIdled(ByVal sender As Object, ByVal e As WorkflowEventArgs)
			Dim watch As Stopwatch = GetTimerByWorkflow(e.WorkflowInstance.InstanceId)
			watch.Stop()
			TraceDT.Add("Global: Workflow instance idled.", e.WorkflowInstance.InstanceId, Me.GetType())
		End Sub

		Private Sub WorkflowRuntime_WorkflowCompleted(ByVal sender As Object, ByVal e As WorkflowCompletedEventArgs)
			Dim watch As Stopwatch = GetTimerByWorkflow(e.WorkflowInstance.InstanceId)
			watch.Stop()
			TraceDT.Add("Global: Workflow instance finished with response {0}. Total 'active' time is {1:N3} seconds.", e.WorkflowInstance.InstanceId, Me.GetType(), e.OutputParameters("EmployeeResponse"), watch.ElapsedMilliseconds / 1000.0)
		End Sub

		Private Sub WorkflowRuntime_WorkflowTerminated(ByVal sender As Object, ByVal e As WorkflowTerminatedEventArgs)
			TraceDT.Add("Global: Workflow has crashed: " & e.Exception.Message, e.WorkflowInstance.InstanceId, Me.GetType())
		End Sub

		Protected Sub Session_Start(ByVal sender As Object, ByVal e As EventArgs)
		End Sub

		Protected Sub Application_BeginRequest(ByVal sender As Object, ByVal e As EventArgs)
		End Sub

		Protected Sub Application_AuthenticateRequest(ByVal sender As Object, ByVal e As EventArgs)
		End Sub

		Protected Sub Application_Error(ByVal sender As Object, ByVal e As EventArgs)
		End Sub

		Protected Sub Session_End(ByVal sender As Object, ByVal e As EventArgs)
		End Sub

		Protected Sub Application_End(ByVal sender As Object, ByVal e As EventArgs)
			TraceDT.Add("Global: Shutting down WorkflowRuntime...", Guid.Empty, Me.GetType())
			WorkflowRuntime.StopRuntime()
			WorkflowRuntime.Dispose()
		End Sub

		Private Sub InstallPersistenceService()
			'
			' Almost certainly would want persistence for spanning worker 
			' process restarts if this were a 'real' application.
			'

			'
			' Run the scripts to create the sql persistence DB at
			'    C:\Windows\Microsoft.NET\Framework\v3.0\Windows Workflow Foundation\SQL\EN
			'      AND
			'    C:\Windows\Microsoft.NET\Framework\v3.5\SQL\EN
			'

			If (Not GetUseDB()) Then
				Return
			End If

			Dim settings As ConnectionStringSettings = ConfigurationManager.ConnectionStrings("WF_Persist")

			Dim sqlSvc As New SqlWorkflowPersistenceService(settings.ConnectionString, True, TimeSpan.FromSeconds(10), TimeSpan.FromSeconds(10))

			WorkflowRuntime.AddService(sqlSvc)
		End Sub

		Private Function GetTimerByWorkflow(ByVal instanceId As Guid) As Stopwatch
			If (Not timers.ContainsKey(instanceId)) Then
				timers(instanceId) = New Stopwatch()
			End If

			Return timers(instanceId)
		End Function

		Private Shared Function GetUseDB() As Boolean
			Return Convert.ToBoolean(ConfigurationManager.AppSettings("usePersistDB"))
		End Function
	End Class
End Namespace