﻿Imports System
Imports System.Workflow.Activities
Imports System.Workflow.Runtime
Imports System.Workflow.ComponentModel

Namespace WorkflowCore
    Partial Public MustInherit Class AsyncActivity
        Inherits SequenceActivity

        Private m_queueName As String

        Protected Property QueueName() As String
            Get
                Return m_queueName
            End Get
            Set(ByVal value As String)
                m_queueName = value
            End Set
        End Property

        Protected Sub New(ByVal queueName As String)
            m_queueName = queueName
            InitializeComponent()
        End Sub

        Protected Overrides Sub Initialize(ByVal provider As System.IServiceProvider)
            Dim svc As WorkflowQueuingService = CType(provider.GetService(GetType(WorkflowQueuingService)), WorkflowQueuingService)

            GetQueueInternal(svc)
            MyBase.Initialize(provider)
        End Sub

        Protected Function GetQueue(ByVal ctx As ActivityExecutionContext) As WorkflowQueue
            Return GetQueueInternal(ctx.GetService(Of WorkflowQueuingService))
        End Function

        Private Function GetQueueInternal(ByVal svc As WorkflowQueuingService) As WorkflowQueue
            If (Not svc.Exists(QueueName)) Then
                Return svc.CreateWorkflowQueue(QueueName, False)
            End If
            Return svc.GetWorkflowQueue(QueueName)
        End Function

        Private m_hasSubscribed As Boolean = False

        Protected Sub SubscribeToItemAvailable(ByVal ctx As ActivityExecutionContext)
            If (m_hasSubscribed) Then
                Return
            End If
            m_hasSubscribed = True
            Dim queue As WorkflowQueue = GetQueue(ctx)
            AddHandler queue.QueueItemAvailable, AddressOf QueueItemAvailable
        End Sub

        Private Sub QueueItemAvailable(ByVal sender As Object, ByVal e As QueueEventArgs)
            Dim ctx As ActivityExecutionContext = CType(sender, ActivityExecutionContext)
            Try
                OnQueueItemAvailable(ctx)
            Finally
                ctx.CloseActivity()
            End Try
        End Sub

        Protected MustOverride Sub OnQueueItemAvailable(ByVal ctx As ActivityExecutionContext)
    End Class
End Namespace
